'--------------------------------------------------------
' Change the display order of selected elements.
'--------------------------------------------------------
'
'  Macro name: ChgOrder.bas
'  Version:    1.10
'  Date:       04.08.1999
'
'--------------------------------------------------------
'  Copyright (1999) Andrei Columban, All rights reserved
'
' Mr. Columban has retired from MicroStation development
' and is persuing a new career path. Please direct
' comments and inquiries to info@cadgurus.com.
'
' Revisions:  Copyright (2002) Buddy Branham, V8 Updates
' All rights reserved.
'                                                                             
' Redistribution and use in source and binary forms, with or without
' modification, are permitted provided that the following conditions
' are met:                                                                    
'                                                                               
'   Redistribution of source code must retain the above copyright 
'   notice, this list of conditions and the following disclaimer. 
'                                                                                           
'   Redistribution in binary form must reproduce the above copyright 
'   notice, this list of conditions and the following disclaimer in 
'   the documentation and/or other materials provided with the 
'   distribution.                                                                      
'                                                                            
'   Neither name of Ramsey Systems, Inc. nor the names of its  
'   contributors may be used to endorse or promote products derived from
'   this software without specific prior written permission.                    
'                                                                                       
' THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
' "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT 
' NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
' FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS
' OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
' SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
' LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF 
' USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
' AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
' OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
' OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
' OF SUCH DAMAGE.                                                                   
'                                                                                                        
'--------------------------------------------------------

Const LF = Chr$(10)

Dim cancelled As Integer
Dim elemSet   As MbeElementSet

'--------------------------------------------------------

Sub processManual ()

    Dim point        As MbePoint
    Dim view         As MbeView
    Dim iView        As Integer
    Dim elementFound As Long
    Dim element      As New MbeElement
    Dim filePos      As Long

    ' Change view attribute FILL to OFF
    For iView = 1 to MBE_MaxViews
        If MbeViews (iView).active <> 0 Then
            Set view = MbeViews(iView)
            view.areaFill = 0
        End If
    Next iView

StartHere:
    ' Display messages
    MbeWriteStatus "ChgOrder -  2002 Columban, Branham, Cadgurus"
    MbeWritePrompt  "Select element!"
    MbeWriteError "(RESET to cancel)"

    ' Select element
    MbeStartLocate 0, 0, 0

    MbeGetInput MBE_DataPointInput, MBE_ResetInput
    If MbeState.inputType = MBE_ResetInput Then
        cancelled = 1
        Goto RestoreViews
    End If

    status = MbeState.getInputDataPoint (point, iview)
    mbeSendLastInput

    If mbeState.cmdResult = MBE_AcceptQuery Then
        elementFound = TRUE     ' found element
        filePos = element.fromLocate ()
        element.display MBE_Hilite

        button = mbeOpenModalDialog (1)

        Select Case button
            Case 2000
                Goto BringToFront
            Case 2001
                MbeState.noElementDisplay = 1
                MbeSendCommand "IUPDATE BOTH"    'update view contents
                MbeSendKeyin "mdl keyin CALCULAT calc mdlSelect_allElements()"
                MbeSendKeyin "mdl keyin CALCULAT calc mdlSelect_removeElement(" + CStr(filePos) + ",mdlModelRef_getActive (),-1)"
            Case 2002
                cancelled = 1
                Goto RestoreViews
        End Select
    End If

    If elementFound = FALSE Then Goto StartHere

BringToFront:
    MbeSendKeyin "WSET ADD"
    if button=2000 then
    ' Send a data point to the current command
    MbeSendDataPoint point, iview
    ' Confirm selection
    MbeSendDataPoint point, iview
    end if
    ' Bring selected element to front
    MbeSendKeyin "WSET DROP"
    MbeState.noElementDisplay = 0

RestoreViews:
    ' Change view attribute FILL to ON
    For iView = 1 to MBE_MaxViews
        If MbeViews (iView).active <> 0 Then
            Set view = MbeViews(iView)
            view.areaFill = 1
        End If
    Next iView

End Sub

'--------------------------------------------------------

Sub processSelSet ()

    Dim elem       As New MbeElement
    Dim elemSet    As New MbeElementSet
    Dim point      As MbePoint
    Dim filePos    As Long
    Dim setMember  As MbeSetMember

    MbeWriteError "Processing Selection Set..."
    MbeWriteStatus "ChgOrder -  1999 Andrei C."
    button = mbeOpenModalDialog (1)

    Select Case button
        Case 2000
            Goto BringToFront
        Case 2001
            MbeState.noElementDisplay = 1
            If elemSet.fromSelectionSet (1) = MBE_Success Then
                status = elemSet.getFirst (setMember)
                MbeSendKeyin "mdl keyin CALCULAT calc mdlSelect_allElements()"
                While status = MBE_Success
                    filePos = elem.fromFile (setMember.filePos, setMember.fileNum)
                    MbeSendKeyin "mdl keyin CALCULAT calc mdlSelect_removeElement(" + CStr(filePos) + ",mdlModelRef_getActive (),-1)"
                    status = elemSet.getNext (setMember)
                Wend
            End If
        Case 2002
            cancelled = 1
            Exit Sub
    End Select

BringToFront:
    MbeSendKeyin "WSET ADD"
    MbeSendKeyin "WSET DROP"
    MbeState.noElementDisplay = 0

    ' clear selection set
    elemSet.clear

End Sub

'--------------------------------------------------------

Sub processFenSet ()

    Dim elem       As New MbeElement
    Dim elemSet    As New MbeElementSet
    Dim point      As MbePoint
    Dim filePos    As Long
    Dim setMember  As MbeSetMember
    Dim iView      As Integer

    MbeWriteError "Processing Fence Set..."
    MbeWriteStatus "ChgOrder -  1999 Andrei C."
    button = mbeOpenModalDialog (1)

    Select Case button
        Case 2000
            Goto BringToFront
        Case 2001
            MbeState.noElementDisplay = 1
            If elemSet.fromFence (1) = MBE_Success Then
                status = elemSet.getFirst (setMember)
                MbeSendKeyin "mdl keyin CALCULAT calc mdlSelect_allElements()"
                While status = MBE_Success
                    filePos = elem.fromFile (setMember.filePos, setMember.fileNum)
                    MbeSendKeyin "mdl keyin CALCULAT calc mdlSelect_removeElement(" + CStr(filePos) + ",mdlModelRef_getActive (),-1)"
                    status = elemSet.getNext (setMember)
                Wend
                selected = 1
            End If
        Case 2002
            cancelled = 1
            Exit Sub
    End Select

BringToFront:
    If selected = 1 Then
        MbeSendKeyin "WSET ADD"
    Else
        MbeSendKeyin "FENCE WSET ADD"
        ' Confirm selection
        MbeSendDataPoint point, iview
    End If
    MbeSendKeyin "WSET DROP"
    MbeState.noElementDisplay = 0

    MbeSendKeyin "place fence"
    MbeSendReset
    ' clear selection set
    elemSet.clear

End Sub

'--------------------------------------------------------

Sub main

    Dim point        As MbePoint
    Dim view         As MbeView
    Dim iView        As Integer
    Dim elementFound As Long
    Dim element      As New MbeElement
    Dim filePos      As Long

    MbeSendCommand "MARK"
    ' If some elements are selected then deselect them
    'MbeSendCommand "CHOOSE NONE"
    ' Turn off all messages except those written by this macro
    MbeSendCommand "NOECHO"

    Set elemSet = New MbeElementSet
    cancelled   = 0
    MbeWriteStatus "ChgOrder -  1999 Andrei C."

    If elemSet.fromSelectionSet() = Mbe_Success Then
        selSet = TRUE
    End If 

    'We use the tcb variable here because elemSet.fromFence clears the active selection set
    If MbeCExpressionDouble("tcb->fence")  Then
        fence = TRUE
    End If

    If selSet AND fence Then    'Both Selection Set and Fence active!
        button = MbeOpenModalDialog (2)

        Select Case button
        Case 1000
            status = elemSet.fromFence()
            If status = 0 Then
                processFenSet
            End If    
        Case 1001
            status = elemSet.fromSelectionSet()
            If status = 0 Then
                processSelSet
            End If
        Case 1002
            cancelled = 1
            Goto ENDMACRO
        End Select
    ElseIf Fence Then    'Fence only active
        status = elemSet.fromFence()
        If status = 0 Then
            processFenSet  'Processing fence set
        End If
    ElseIf selSet Then    'Selection set only active    
        status = elemSet.fromSelectionSet()
        If status = 0 Then
            processSelSet  'Processing selection set
        End If
    Else
        finished = FALSE
        button = MbeMessageBox ("No Selection Set or Fence defined !" + LF + LF + _
                "  Process the elements manually?", MBE_YesNoBox or MBE_QuestionIcon)
        While button <> MBE_BUTTON_NO and NOT finished
            If button = MBE_BUTTON_YES Then
                processManual  'Processing elements manually
            finished = TRUE
            End If 
        Wend
    End If

ENDMACRO:
    MbeSendCommand "UPDATE BOTH"    'update view contents
    If cancelled = 1 Then
        MbeWriteStatus "Program cancelled!"
    Else
        MbeWriteStatus "Program finished!"
    End If
    MbeWriteError
    MbeSendCommand "ECHO"
    MbeStartDefaultCommand

End Sub
 
 
